package BaseUtils;

use v5.36.0;

use strict;
use warnings;
use utf8;

use Moo;
use Path::Tiny;

sub install ( $self, $source, $dest ) {
    if ( -e $dest
        && _getSha512SumFile($dest) eq _getSha512SumFile($source) )
    {
        say "$source is already installed, skipping.";
        return;
    }
    say "Target $source differs, installing...";
    my $return_code = system 'sudo', 'cp', '-L', $source, $dest;
    if ( $return_code != 0 ) {
        die "Unable to install $dest";
    }
    return 1;
}

sub _getSha512SumFile ($file) {
    open my $fh, '-|', 'sha512sum', $file;
    my $text = <$fh>;
    close $fh;

    my ($sha512sum) = $text =~ /^(\S+)/;
    return $sha512sum;
}

sub forkWait($self) {
    my $pid = fork;
    if ($pid) {
        return sub {
            wait;
            my $return_code = $?;
            return $return_code;
        };
    }
    return;
}

sub getVmlinuz ($self, $source) {
    my $boot = $source->child('boot');
    open my $fh, '-|', 'ls', '-t', $boot;
    while ( my $file = <$fh> ) {
        chomp($file);
        if ( $file =~ /^vmlinuz/ ) {
            return $file;
        }
    }
    die 'Unable to find vmlinuz.';
}

sub chroot($self, $target) {
    chroot "$target";
    chdir '/';
}

{
    no feature 'signatures';
    sub openWriteCommandChroot($\$$@) {
        my $self = shift;
        my $fh = shift;
        pipe my ($read), $$fh;
        my $parent_subroutine = $self->forkWait();
        if (defined $parent_subroutine) {
            close $read;
            return $parent_subroutine;
        }
        close $$fh;
        my $target = shift;
        $self->chroot($target);
        my @command = @_;
        open my $inner_fh, '|-', @command;
        while (my $line = <$read>) {
            chomp $line;
            if (!length $line > 0) {
                next;
            }
            say $inner_fh $line;
        }
        close $inner_fh;
        my $return_code = $?;
        exit $?;
    }
}
sub execChroot($self, $target, @command) {
    my $parent_subroutine = $self->forkWait();
    if (defined $parent_subroutine) {
        return $parent_subroutine->();
    }
    $self->chroot($target);
    my $return_code = system @command;
    exit $return_code;
}

sub mountType ( $self, $fs_type, $fs ) {
    if($self->checkMounted($fs)) {
        say "$fs already mounted";
        return;
    }
    my $return_code = system 'sudo', 'mount', '-t', $fs_type, $fs_type, $fs;
    if ( $return_code != 0 ) {
        die "Unable to mount $fs.";
    }
}

sub checkMounted ($self, $fs) {
    $fs = $fs->realpath;
    my $return = index( $self->getMounts(), $fs ) != -1;
    say "$fs is mounted." if $return;
    return $return;
}

sub mountRbind ( $self, $source, $fs ) {
    if($self->checkMounted($fs)) {
        say "$fs already mounted";
        return;
    }
    my $return_code = system 'sudo', 'mount', '--rbind', $source, $fs;
    if ( $return_code != 0 ) {
        die "Unable to mount $fs.";
    }
}

sub readPackagesToInstall {
    my @packages;
    open my $fh, '<', path(__FILE__)->parent->parent->child('packages');
    while ( my $line = <$fh> ) {
        chomp $line;
        push @packages, $line;
    }
    close $fh;
    return \@packages;
}

sub readBuildPackagesToInstall {
    my @packages;
    open my $fh, '<', path(__FILE__)->parent->parent->child('build_packages');
    while ( my $line = <$fh> ) {
        chomp $line;
        push @packages, $line;
    }
    close $fh;
    return \@packages;
}

sub createBasicMountsChroot ($self, $target) {
    my $dest_proc = $target->child('proc');
    my $dest_dev  = $target->child('dev');
    my $dest_sys  = $target->child('sys');
    my $dest_run  = $target->child('run');

    $dest_proc->mkpath;
    $dest_dev->mkpath;
    $dest_sys->mkpath;
    $dest_run->mkpath;

    $self->mountType( 'proc',  $dest_proc );
    system 'mount', '--make-slave', $dest_proc;
    $self->mountType( 'sysfs', $dest_sys );
    system 'mount', '--make-slave', $dest_sys;
    $self->mountRbind( '/dev', $dest_dev );
    system 'mount', '--make-slave', $dest_dev;
    $self->mountRbind( '/run', $dest_run );
    system 'mount', '--make-slave', $dest_run;
}
{
    my $mounts;

    sub getMounts {
        if ( !defined $mounts ) {
            open my $fh, '-|', 'sudo', 'mount';
            $mounts = join '', <$fh>;
            close $fh;
        }
        return $mounts;
    }
}

sub _installRootFiles ( $self, $target, $files_dir ) {
    my $stage_root_source = path($files_dir);
    $stage_root_source->visit(
        sub {
            my ( $path, $state ) = @_;
            if ( !-f $path ) {
                return;
            }
            my $target_path = $target->child( $path =~ s{^$files_dir/*}{}r );
            $target_path->parent->mkpath;
            my $basename = $path->basename;
            return if $basename eq '.exists';
            say "Installing $path.";
            $self->install( $path, $target_path );
        },
        { recurse => 1 }
    );
}
1;
